package algo.tp.gui;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import algo.tp.model.BasisCell;
import algo.tp.model.CycleElement;
import algo.tp.service.TPException;
import algo.tp.service.TPService;

public class TPVisualizer extends JFrame implements ActionListener {
	
	TPService service;

	private int step = 0;
	
	private boolean completed = false;
	
	private JPanel panel = new JPanel();

	private JPanel matrixPanel = new JPanel();

	private JPanel titel = new JPanel();

	private JPanel east = new JPanel();

	private JPanel west = new JPanel();

	private JPanel south;

	private JLabel label_tpMatrix = new JLabel("Transportmatrix");
	
	private JLabel indicator = new JLabel();

	private JButton weiter = new JButton("Go");

	private JButton start = new JButton("Nord-West-Winkel Verfahren");
	
	private Font font = new Font("Helvetica", Font.BOLD, 18);

	private JPanel out;

	private Vector tableCells;

	private TableCell getTableCell(int row, int collumn) {
		int k = (row * service.getDestinations().getCount() + collumn);
		return (TableCell) tableCells.get(k);
	}
	
	private String indicatorString() {
		StringBuffer sb = new StringBuffer();
		if(completed) {
			sb.append("   Die Verteilung ist optimal!!!   ");
			sb.append(System.getProperty("line.separator"));
			sb.append(" Keine negative PseudoPices mehr!!!  ");
			sb.append("Transportkosten = " + service.getBasis().getPrice());
		} else {
			sb.append("   Die Verteilung ist nicht optimal!   Transportkosten = " + service.getBasis().getPrice());
		}
		return sb.toString();
	}

	public TPVisualizer(TPService service) {
		super();
		this.service = service;
		addWindowListener(new WindowClosingAdapter(true));
	}

	public void beforStart() {
		service.initBasisNorthWest();
		try {
			service.calculatePotentials();
			Vector vv = service.negativePseudoPrices();
			service.selectOptimalCycle(vv);
		} catch (Exception em) {
			em.printStackTrace();
		}
	}

	private boolean isFirst(int i, int j) {
		boolean b = false;
		CycleElement first = service.getStart();
		if (first != null) {
			if ((first.getBasisCell().getColumn() == j)
					&& (first.getBasisCell().getRow() == i)) {
				b = true;
			}
		}
		return b;
	}

	private void initMatrix(int rows, int collumns) {

		GridLayout grid = new GridLayout(rows + 1, collumns + 1);
		tableCells = new Vector(rows * collumns);
		out = new JPanel(grid);
		HeadlineTableCell headline = null;
		TableCell z = null;
		JLabel freeLabel = new JLabel("");
		out.add(freeLabel);
		for (int c = 1; c <= collumns; c++) {
			headline = new HeadlineTableCell("           "
					+ service.getDestinations().getAmount(c),
					"/images/destination.gif", "destination");
			out.add(headline);
		}
		for (int i = 0; i < rows; i++) {
			headline = new HeadlineTableCell(""
					+ (service.getSources().getAmount(i + 1)) + "   ",
					"/images/source.gif", "source");
			out.add(headline);
			for (int j = 1; j <= collumns; j++) {
				int price = service.getPrices().get(i + 1, j);
				z = new TableCell(price);
				tableCells.add(z);
				out.add(z);
			}

		}

	}

	public void updateMatrix(int rows, int collumns) {
		indicator.setText(indicatorString());
		TableCell z = null;
		for (int i = 0; i < rows; i++) {
			for (int j = 0; j < collumns; j++) {
				boolean toBasis = false;
				int amount = 0;
				int pPrice = service.getDestinations().getPotential(j + 1);
				pPrice += service.getSources().getPotential(i + 1);
				BasisCell cell = service.getBasis().find(i + 1, j + 1);
				if (cell != null) {
					toBasis = true;
					amount = cell.getAmount();
				}
				z = getTableCell(i, j);
				z.reinit(amount, pPrice, toBasis, isFirst(i + 1, j + 1));
			}
		}
	}

	public void initializeFrame() {


		this.addWindowListener(new WindowClosingAdapter(true));
		this.setLocation(100, 100);
		this.setSize(600, 500);

		label_tpMatrix.setFont(font);

		weiter.addActionListener(this);
		weiter.setActionCommand("weiter");

		start.addActionListener(this);
		start.setActionCommand("start");

		titel.add(label_tpMatrix);
		panel.setLayout(new BorderLayout());
		GridLayout southLayout = new GridLayout(2, 1);
		south = new JPanel(southLayout);
//		south.add(start);
		south.add(indicator);
		south.add(weiter);
		panel.add(titel, BorderLayout.NORTH);
		initMatrix(service.getSources().getCount(), service.getDestinations()
				.getCount());

		matrixPanel.add(out);
		panel.add(matrixPanel, BorderLayout.CENTER);
		panel.add(east, BorderLayout.EAST);
		panel.add(west, BorderLayout.WEST);
		panel.add(south, BorderLayout.SOUTH);
		this.getContentPane().add(panel);
		this.setVisible(true);

	}

	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getActionCommand().equals("start")) {
			service.initBasisNorthWest();
			updateMatrix(service.getSources().getCount(), service
					.getDestinations().getCount());

		}

		if (arg0.getActionCommand().equals("weiter")) {
			boolean b = false;
			if(step == 0) {
				step++;
			} else if (!completed){
				try {
					b = service.iterate();
				} catch (TPException tpe) {
					tpe.printStackTrace();
				}
			} else {
				b = true;
			}
			updateMatrix(service.getSources().getCount(), service
					.getDestinations().getCount());
			completed = b;
			out.repaint();
		}
	}

}
