package algo.tp.model;

import algo.tp.service.TPException;

public class Basis {
	private BasisCell[] cells;

	private int count;

	private int running;

	private int rows;

	private int columns;

	public Basis(int rows, int columns) {
		this.rows = rows;
		this.columns = columns;
		this.running = 0;
		this.count = rows + columns - 1;
		cells = new BasisCell[count];
	}

	public BasisCell find(int row, int column) {
		int k = 0;
		boolean found = false;
		for (; k < count; k++) {
			if ((cells[k].getRow() == row) && (cells[k].getColumn() == column)) {
				found = true;
				break;
			}
		}
		if (found) {
			return cells[k];
		} else {
			return null;
		}
	}

	public BasisCell findByRow(int number) {
		int k = 0;
		boolean found = false;
		for (; k < count; k++) {
			if ((cells[k] != null) && (cells[k].getRow() == number)) {
				found = true;
				break;
			}
		}
		if (found) {
			return cells[k];
		} else {
			return null;
		}
	}

	public BasisCell findByColumn(int number) {
		int k = 0;
		boolean found = false;
		for (; k < count; k++) {
			if ((cells[k] != null) && (cells[k].getColumn() == number)) {
				found = true;
				break;
			}
		}
		if (found) {
			return cells[k];
		} else {
			return null;
		}
	}

	public int put(BasisCell newBasisCell) {
		if (running < count) {
			cells[running] = newBasisCell;
			running++;
		}
		return running;

	}

	private int remove(BasisCell toRemove) throws TPException {
		int row = toRemove.getRow();
		int column = toRemove.getColumn();
		int k = 0;
		BasisCell b = null;
		boolean found = false;
		for (; k < count; k++) {
			if ((cells[k].getRow() == row) && (cells[k].getColumn() == column)) {
				found = true;
				break;
			}
		}
		if (found) {
			//Vertical-Kette schliessen;
			b = toRemove.getVerticalNeighbour();
			if (b != null) {
				while (!toRemove.equals(b.getVerticalNeighbour())) {
					b = b.getVerticalNeighbour();
				}
				if (b.equals(toRemove.getVerticalNeighbour())) {
					b.setVerticalNeighbour(null);
				} else {

					b.setVerticalNeighbour(toRemove.getVerticalNeighbour());
				}
			}
			//Horizontal-Kette schliessen;
			b = toRemove.getHorizontalNeighbour();
			if (b != null) {
				while (!toRemove.equals(b.getHorizontalNeighbour())) {
					b = b.getHorizontalNeighbour();
				}
				if (b.equals(toRemove.getHorizontalNeighbour())) {
					b.setHorizontalNeighbour(null);
				} else {

					b.setHorizontalNeighbour(toRemove.getHorizontalNeighbour());
				}
			}
			cells[k] = null;
		} else {
			throw new TPException("Basis Element Not Found");
		}
		return k;
	}

	public void replace(BasisCell toRemove, BasisCell cell) throws TPException {
		int k = remove(toRemove);
		//Vertical-Kette schliessen;
		BasisCell b = findByColumn(cell.getColumn());
		BasisCell b1 = null;
		if (b != null) {
			b1 = b.getVerticalNeighbour();
			b.setVerticalNeighbour(cell);
			if (b1 == null) {
				cell.setVerticalNeighbour(b);
			} else {
				cell.setVerticalNeighbour(b1);
			}
		}
		//Horizontal-Kette schliessen;
		b = findByRow(cell.getRow());
		if (b != null) {
			b1 = b.getHorizontalNeighbour();
			b.setHorizontalNeighbour(cell);
			if (b1 == null) {
				cell.setHorizontalNeighbour(b);
			} else {
				cell.setHorizontalNeighbour(b1);
			}

		}
		cell.setPseudoPrice(0);//Pseudopreis von BAsiszellen ist immer =0!
		cells[k] = cell;
	}

	public int getCount() {
		return count;
	}

	public int getPrice() {
		int sum = 0;
		//		boolean found = false;
		for (int k = 0; k < count; k++) {
			sum = sum + cells[k].getPrice() * cells[k].getAmount();
		}
		return sum;
	}

	public void printBasis() {
		int b[][] = new int[rows][columns];
		for (int i = 0; i < rows; i++) {
			for (int j = 0; j < columns; j++) {
				if (find(i+1, j+1) != null) {
					b[i][j] = find(i+1, j+1).getAmount();
					System.out.print(b[i][j] + "   ");
				} else {
					b[i][j] = 0;
					System.out.print(b[i][j] + "    ");
				}
			}
			System.out.println("");
		}
		//		return b;
	}
}