package algo.tp.service;

import java.util.Vector;

import algo.tp.model.Basis;
import algo.tp.model.BasisCell;
import algo.tp.model.CycleElement;
import algo.tp.model.Prices;
import algo.tp.model.StoreAmounts;

public class TPService {

	private CycleElement start;
	
	private Basis basis;

	private StoreAmounts sources;

	private StoreAmounts destinations;

	private Prices prices;

	private Vector visited;

	public Basis getBasis() {
		return basis;
	}
	
	public void setBasis(Basis basis) {
		this.basis = basis;
	}

	public StoreAmounts getDestinations() {
		return destinations;
	}

	public void setDestinations(StoreAmounts destinations) {
		this.destinations = destinations;
	}

	public Prices getPrices() {
		return prices;
	}

	public void setPrices(Prices prices) {
		this.prices = prices;
	}

	public StoreAmounts getSources() {
		return sources;
	}

	public void setSources(StoreAmounts sources) {
		this.sources = sources;
	}

	public void initBasisNorthWest() {
		basis = new Basis(sources.getCount(), destinations.getCount());
		BasisCell first = new BasisCell(1, 1);
		BasisCell running = first;
		BasisCell next = null;
		boolean vertical = false;
		int i = 1;
		int j = 1;
		int amount = 0;
		int remainder = sources.getAmount(1);
		int p = 0;
		if (sources.getAmount(1) < destinations.getAmount(1)) {
			vertical = !vertical;
			remainder = destinations.getAmount(1);
			amount = sources.getAmount(1);
		} else {
			amount = destinations.getAmount(1);
		}
		first.setAmount(amount);
		first.setPrice(prices.get(1, 1));
		int k = basis.put(first);
		remainder = remainder - amount;
		while (k < basis.getCount()) {
			if (vertical) {
				i++;
				next = new BasisCell(i, j);
				p = sources.getAmount(i);
				running.setVerticalNeighbour(next);
			} else {
				j++;
				next = new BasisCell(i, j);
				p = destinations.getAmount(j);
				running.setHorizontalNeighbour(next);
			}
			amount = (remainder > p) ? p : remainder;
			next.setAmount(amount);
			next.setPrice(prices.get(i, j));
			k = basis.put(next);
			remainder = remainder - amount;
			running = next;
			if (remainder == 0) {
				vertical = !vertical;
				if (vertical) {
					//Kette schlissen
					running.setHorizontalNeighbour(first);
					//Rest neu definieren
					remainder = destinations.getAmount(j) - running.getAmount();
				} else {
					running.setVerticalNeighbour(first);
					remainder = sources.getAmount(i) - running.getAmount();
				}
				first = running;
			}
		}
	}

	//	public void initBasisMinElement() {
	//
	//	}

	private void calculateDesitnations(int index) throws TPException {

		int p1 = sources.getPotential(index);
		BasisCell b1 = basis.findByRow(index);
		int j = 0;
		BasisCell b = b1;
		do {
			j = b.getColumn();
			if (!destinations.isDefined(j)) {
				destinations.setPotential(j, prices.get(index, j) - p1);
			} else if (
				destinations.getPotential(j) + p1 != prices.get(index, j)) {
				throw new TPException("Error calculate potentials");
			}
			b = b.getHorizontalNeighbour();
		} while ((b != null) && (!b.equals(b1)));
	}

	private void calculateSources(int index) throws TPException {
		int p1 = destinations.getPotential(index);
		BasisCell b1 = basis.findByColumn(index);
		int i = 0;
		BasisCell b = b1;
		do {
			i = b.getRow();
			if (!sources.isDefined(i)) {
				sources.setPotential(i, prices.get(i, index) - p1);
			} else if (sources.getPotential(i) + p1 != prices.get(i, index)) {
				throw new TPException("Error calculate potentials");
			}
			b = b.getVerticalNeighbour();
		} while ((b != null) && (!b.equals(b1)));
	}

	public void calculatePotentials() throws TPException {
		sources.reset();
		destinations.reset();
		// Ein Potential ist freiwaehlend,
		// da die Zahl der Basiselemente ist gleich m+n-1
		sources.setPotential(1, 0);
		while (!(sources.allDefined() && destinations.allDefined())) {
			for (int i = 1; i <= sources.getCount(); i++) {
				if (sources.isDefined(i)) {
					calculateDesitnations(i);
				}
			}
			for (int j = 1; j <= destinations.getCount(); j++) {
				if (destinations.isDefined(j)) {
					calculateSources(j);
				}
			}
		}
	}

	public Vector negativePseudoPrices() {
		Vector v = new Vector();
		int p1 = sources.getPotential(1);
		int p2 = destinations.getPotential(1);
		BasisCell cell;
		for (int i = 1; i <= sources.getCount(); i++) {
			p1 = sources.getPotential(i);
			for (int j = 1; j <= destinations.getCount(); j++) {
				p2 = destinations.getPotential(j);
				int p = prices.get(i, j) - (p1 + p2);
				if (p < 0) {
					cell = new BasisCell(i, j);
					cell.setPseudoPrice(p);
					v.add(cell);
				}
			}
		}
		return v;
	}

	public CycleElement buildCycle(int row, int column) throws TPException {
		if (basis == null) {
			throw new TPException("Basis not initialised");
		} else if (basis.find(row, column) != null) {
			throw new TPException("Field not empty");
		}

		CycleElement result = null;
		visited = new Vector();
		boolean vertical = true;
		BasisCell b = basis.findByColumn(column);//erstes Element im Zyklus
		while ((b != null) && !visited.contains(b) && (result == null)) {
			result = buildCycle(!vertical, b.getHorizontalNeighbour(), row);
			if (result != null) {
				result = new CycleElement(b, result);
			} else {
				visited.add(b);
				b = b.getVerticalNeighbour();
			}
		}
		return result;
	}

	private CycleElement buildCycle(
		boolean vertical,
		BasisCell b,
		int targetRow) {
		CycleElement result = null;
		BasisCell b1 = null;
		if ((b == null) || visited.contains(b)) {
			return null;
		} else if (targetRow == b.getRow()) {
			//Letztes Element:
			return new CycleElement(b);
		}
		if (vertical) {
			while ((b != null) && !visited.contains(b)) {
				result =
					buildCycle(
						!vertical,
						b.getHorizontalNeighbour(),
						targetRow);
				if (result != null) {
					return new CycleElement(b, result);
				} else {
					visited.add(b);
					b = b.getVerticalNeighbour();
				}
			}
		} else {
			b1 = b.getVerticalNeighbour();
			//'Ziel erreicht' prfen
			while ((b1 != null) && (b1 != b)) {
				if (targetRow == b1.getRow()) {
					//Letztes Element:
					CycleElement last = new CycleElement(b1);
					return new CycleElement(b, last);
				}
				b1 = b1.getVerticalNeighbour();
			}
			while ((b != null) && !visited.contains(b)) {
				result =
					buildCycle(!vertical, b.getVerticalNeighbour(), targetRow);
				if (result != null) {
					return new CycleElement(b, result);
				} else {
					visited.add(b);
					b = b.getHorizontalNeighbour();
				}
			}
		}

		return null;
	}

	public void selectOptimalCycle(Vector v) throws TPException {
		CycleElement result = null;
		CycleElement first = null;
		CycleElement next = null;
		int profit = 0;
		for (int k = 0; k < v.size(); k++) {
			BasisCell b = (BasisCell) v.get(k);
			//Zyklus fr das leere Feld bauen
			first = buildCycle(b.getRow(), b.getColumn());
			//Maximal-mgliches Amount definieren
			next = first;
			int amount = first.getAmount();
			boolean negative = true;
			while (next != null) {
				int tmp = next.getAmount();
				if ((negative) && (tmp < amount)) {
					amount = tmp;
				}
				next = next.getNext();
				negative = !negative;
			}
			//setzen
			b.setAmount(amount);
			b.setPrice(prices.get(b.getRow(), b.getColumn()));
			if (amount * b.getPseudoPrice() <= profit) {
				profit = amount * b.getPseudoPrice();
				result = new CycleElement(b, first);
			}
		}
		this.start = result;
	}

	public boolean iterate() throws TPException {
		boolean completed = false;
		CycleElement next = null;
		BasisCell toRemove = null;
		//Verschieben
		next = start.getNext();
		int amount = start.getAmount();
		while (next != null) {
			int newAmount = next.getAmount() - amount;
			next.setAmount(newAmount);
			if (newAmount == 0) {
				toRemove = next.getBasisCell();
			}
			next = next.getNext();
			amount = -amount;
		}
		basis.replace(toRemove, start.getBasisCell());
		calculatePotentials();
		Vector v = negativePseudoPrices();
		selectOptimalCycle(v);
		if(v.isEmpty()) {
			completed = true;
		}
		return completed;
	}

	public CycleElement getStart() {
		return start;
	}
}